/* ld script to make ARM Linux kernel
 * taken from the i386 version by Russell King
 * Written by Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 */
OUTPUT_ARCH(arm)
ENTRY(stext)
SECTIONS
{
	. = 0xC0008000;
	.init : {			/* Init code and data		*/
		_stext = .;
		__init_begin = .;
			*(.text.init)
		__proc_info_begin = .;
			*(.proc.info)
		__proc_info_end = .;
		__arch_info_begin = .;
			*(.arch.info)
		__arch_info_end = .;
		__tagtable_begin = .;
			*(.taglist)
		__tagtable_end = .;
			*(.data.init)
		. = ALIGN(16);
		__setup_start = .;
			*(.setup.init)
		__setup_end = .;
		__initcall_start = .;
			*(.initcall.init)
		__initcall_end = .;
		. = ALIGN(4096);
		__init_end = .;
	}

	/DISCARD/ : {			/* Exit code and data		*/
		*(.text.exit)
		*(.data.exit)
		*(.exitcall.exit)
	}

	.text : {			/* Real text segment		*/
		_text = .;		/* Text and read-only data	*/
		/* add here the hot functions */
			*(.text.mv_gtw_tx)
			*(.text.ipt_do_table)
			*(.text.do_gettimeofday)
			*(.text.ip_ct_refresh)
			*(.text.mv_gtw_poll)
                        *(.text.arm926_dma_inv_range)
			*(.text.cpu_arm926_dcache_invalidate_range)
			*(.text.nf_iterate)
			*(.text.ip_finish_output2)
			*(.text.ip_conntrack_in)
			*(.text.dev_queue_xmit)
                        *(.text.mvEthPortTxDone)
                        *(.text.mvEthPortTx)
			*(.text.mvEthPortRx)
			*(.text.netif_receive_skb)
			*(.text.ip_route_input)
			*(.text.pfifo_fast_enqueue)
			*(.text.nf_hook_slow)
			*(.text.alloc_skb)
                        *(.text.net_tx_action)
			*(.text.kfree_skbmem)
			*(.text.ip_rcv)
			*(.text._text)
			*(.text.__umodsi3)
			*(.text.__ip_conntrack_find)
			*(.text.do_bindings)
			*(.text.__kfree_skb)
			*(.text.qdisc_restart)
			*(.text.rt_hash_code)
			*(.text.ip_forward)
			*(.text.memmove)
			*(.text.ip_nat_fn)
			*(.text.ip_forward_finish)
			*(.text.ip_conntrack_find_get)
			*(.text.ipt_hook)
			*(.text.ip_rcv_finish)
			*(.text.eth_type_trans)
			*(.text.ip_refrag)
			*(.text.ipt_route_hook)
			*(.text.skb_release_data)
			*(.text.kmalloc)
                        *(.text.__kmalloc)
			*(.text._kmalloc)
                        *(.text.arm926_dma_flush_range)
			*(.text.pfifo_fast_dequeue)
			*(.text.local_bh_enable)
			*(.text.mvEthPortRxDone)
			*(.text.generic_pkt_to_tuple)
			*(.text.memcmp)
			*(.text.mv_gtw_rx_fill)
			*(.text.ip_finish_output)
			*(.text.ip_nat_out)
			*(.text.ip_confirm)
			*(.text.__memzero)
			*(.text.hash_conntrack)
			*(.text.del_timer)
			*(.text.established)
			*(.text.__ip_ct_find_proto)
			*(.text.neigh_resolve_output)
			*(.text.cpu_arm926_cache_clean_invalidate_range)
			*(.text.ip_conntrack_get)
			*(.text.consistent_sync)
			*(.text.default_idle)
			*(.text.mvEthRxPolicyGet)
			*(.text.ip_ct_find_proto)
			*(.text.mvOsIoVirtToPhy)
			*(.text.mvOsCacheInvalidate)
			*(.text.eth_header)
			*(.text.kfree)
			*(.text.kmem_cache_alloc)
			*(.text.kmem_cache_free)
			*(.text.arm926_cache_fns)
			*(.text.__ip_conntrack_confirm)
			*(.text.set_bit)
			*(.text.add_timer)
			*(.text.ip_conntrack_local)
			*(.text.net_rx_action)
			*(.text.v4wb_clear_user_page)
			*(.text.pfifo_fast_requeue)
			*(.text.nf_register_queue_handler)
			*(.text.schedule)
			*(.text.__page_cache_release)
                        *(.text.do_DataAbort)
			*(.text.d_alloc)
                        *(.text.cond_resched)
			*(.text.__ashrdi3)
                        *(.text.arm926_dma_clean_range)
                        *(.text.unmap_vmas)
                        *(.text.set_termios)
			*(.text.ksoftirqd)
			*(.text.ip_output)
			*(.text.generic_invert_tuple)
			*(.text.do_softirq)
			*(.text.c2u_0cpy8lp)
			*(.text.timer_bh)
			*(.text.memset)
			*(.text.bh_action)
			*(.text.__rpc_execute)
			*(.text.__neigh_event_send)
                	*(.text.*)
                        *(.text)
                	*(.fixup)
			*(.gnu.warning)
			*(.rodata)
			*(.rodata.*)
			*(.glue_7)
			*(.glue_7t)
		*(.got)			/* Global offset table		*/

		_etext = .;		/* End of text section		*/
	}

	.kstrtab : { *(.kstrtab) }

	. = ALIGN(16);
	__ex_table : {			/* Exception table		*/
		__start___ex_table = .;
			*(__ex_table)
		__stop___ex_table = .;
	}

	__ksymtab : {			/* Kernel symbol table		*/
		__start___ksymtab = .;
			*(__ksymtab)
		__stop___ksymtab = .;
	}

	. = ALIGN(8192);

	.data : {
		/*
		 * first, the init task union, aligned
		 * to an 8192 byte boundary.
		 */
		*(.init.task)

		/*
		 * then the cacheline aligned data
		 */
		. = ALIGN(32);
		*(.data.cacheline_aligned)

		/*
		 * and the usual data section
		 */
		*(.data)
		CONSTRUCTORS

		_edata = .;
	}

	.bss : {
		__bss_start = .;	/* BSS				*/
		*(.bss)
		*(COMMON)
		_end = . ;
	}
					/* Stabs debugging sections.	*/
	.stab 0 : { *(.stab) }
	.stabstr 0 : { *(.stabstr) }
	.stab.excl 0 : { *(.stab.excl) }
	.stab.exclstr 0 : { *(.stab.exclstr) }
	.stab.index 0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment 0 : { *(.comment) }
}
