/*
 * Copyright (c) 2000-2002 by Dima Epshtein
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define DRIVER_VERSION "2004-Dec-29"
#define DRIVER_AUTHOR "Dima Epshtein"
#define DRIVER_DESC "USB 2.0 'Enhanced' Host Controller (EHCI) Driver"

#include <linux/config.h>


#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/reboot.h>
#include <linux/usb.h>
#include <linux/moduleparam.h>
/*#include <linux/dma-mapping.h>*/

#ifdef CONFIG_USB_DEBUG
    #define DEBUG
#else
    #undef DEBUG
#endif


#include "../hcd.h"
/*#include "ehci.h"*/

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/unaligned.h>

extern const struct hc_driver   ehci_driver;
extern struct usb_operations    hcd_operations;
extern struct list_head hcd_list;
extern struct semaphore hcd_list_lock;

extern void hcd_irq (int irq, void *__hcd, struct pt_regs * r);

static struct usb_hcd          *marvell_hcd = NULL; 

static char usb_dev_name[]  = "mv_88f5x8x_gadget";
static char usb_host_name[] = "mv_88f5x8x_ehci";
static char usb_bus_name[]  = "mv_platform"; 

#define MV_EHCI_REGS_BASE(dev)  (((dev) == 0) ? 0xF1050100 : 0xF10A0100) 

void  ehci_marvell_hcd_remove (struct usb_hcd *hcd) 
{  
	struct usb_device	*hub;

    if (in_interrupt ()) 
        BUG (); 
  
    hub = hcd->bus->root_hub;
	hcd->state = USB_STATE_QUIESCING;
  	usb_disconnect (&hub);

    hcd->driver->stop (hcd); 
 
    free_irq (hcd->irq, hcd); 
 
   	down (&hcd_list_lock);
	list_del (&hcd->hcd_list);
	up (&hcd_list_lock);

	usb_deregister_bus (hcd->bus);
	usb_free_bus (hcd->bus);
	hcd->bus = NULL;

	hcd->driver->hcd_free (hcd);
} 
 
static int ehci_marvell_probe(const struct hc_driver  *driver) 
{ 
  	struct usb_bus		    *bus;
    int                     retval, dev; 

    printk("ehci_marvell_probe: driver=%p\n", driver);

    dev = 0;
    marvell_hcd = driver->hcd_alloc (); 
    if (marvell_hcd == NULL) 
    { 
        printk("%s: hcd_alloc failed\n", __FUNCTION__); 
        return -ENOMEM; 
    } 
    printk("hcd allocated: marvell_hcd=%p\n", marvell_hcd);
 
    marvell_hcd->driver = (struct hc_driver *) driver; 
    marvell_hcd->description = driver->description; 
    marvell_hcd->irq = IRQ_USB_CTRL(dev); 
    marvell_hcd->regs = (void*)MV_EHCI_REGS_BASE(dev);

    printk("mv_ehci_%d: IRQ=%d, Regs=%p\n", dev, marvell_hcd->irq, marvell_hcd->regs);

   	if (request_irq (marvell_hcd->irq, hcd_irq, SA_SHIRQ, 
                     marvell_hcd->description, marvell_hcd) != 0) {
		printk("request_irq for %d is failed\n", marvell_hcd->irq);
		driver->hcd_free (marvell_hcd);
        return -EBUSY;
	}

    bus = usb_alloc_bus (&hcd_operations);
	if (bus == NULL) {
		dbg ("usb_alloc_bus fail");
		free_irq (marvell_hcd->irq, marvell_hcd);
		driver->hcd_free (marvell_hcd);
		return -ENOMEM;
	}
    printk("mv_ehci - Bus allocated: bus=%p\n", bus);

	marvell_hcd->bus = bus;
	bus->bus_name = usb_bus_name;
	marvell_hcd->product_desc = usb_host_name;
	bus->hcpriv = (void *) marvell_hcd;

	INIT_LIST_HEAD (&marvell_hcd->dev_list);
	INIT_LIST_HEAD (&marvell_hcd->hcd_list);

	down (&hcd_list_lock);
	list_add (&marvell_hcd->hcd_list, &hcd_list);
	up (&hcd_list_lock);

	usb_register_bus (bus);
    printk("mv_ehci - Bus registered: bus=%p\n", bus);

    if ((retval = driver->start (marvell_hcd)) < 0)  
    { 
        ehci_marvell_hcd_remove(marvell_hcd); 
        return retval; 
    } 
    printk("mv_ehci - Started\n");

    return 0; 
} 
 
 
#define DRIVER_INFO DRIVER_VERSION " " DRIVER_DESC

MODULE_DESCRIPTION (DRIVER_INFO);
MODULE_AUTHOR (DRIVER_AUTHOR);
MODULE_LICENSE ("GPL");

static int __init ehci_marvell_init (void) 
{ 
    int status; 

    printk("EHCI Marvell Init\n");
 
    status = ehci_marvell_probe(&ehci_driver); 

    return status;
}
module_init (ehci_marvell_init);

static void __exit ehci_marvell_cleanup (void) 
{   
    ehci_marvell_hcd_remove (marvell_hcd);     
}
module_exit (ehci_marvell_cleanup);
